/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 */
package com.jcg;

import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;
import jakarta.persistence.PersistenceConfiguration;
import jakarta.persistence.TypedQuery;
import java.sql.Connection;
import java.util.List;

/**
 *
 * @author omozegieaziegbe
 */
public class JakartaPersistence32Examples {

    static EntityManagerFactory emf
            = new PersistenceConfiguration("CustomerPU").
                    jtaDataSource("java:global/jdbc/CustomerData")
                    .managedClass(Customer.class)
                    .property(PersistenceConfiguration.LOCK_TIMEOUT, 5000)
                    .createEntityManagerFactory();

    public static void main(String[] args) {

        emf.getSchemaManager().create(true);

        // Perform a write operation using runInTransaction
        emf.runInTransaction(em -> em.runWithConnection(connection -> {
            try (var stmt = ((Connection) connection).createStatement()) {
                stmt.execute("INSERT INTO customer (name, email) VALUES ('John Doe', 'john.doe@example.com')");
            } catch (Exception e) {
                System.out.println("JDBC operation failed: " + e.getMessage());
            }
        }));

        // Perform a read operation using callInTransaction
        Customer customer1 = emf.callInTransaction(em -> em.find(Customer.class, 1L));

        // Run a query to fetch a customer with the name 'John Doe' using streamlined syntax
        Customer customer = emf.callInTransaction(em -> em.createQuery("from Customer where name = 'John Doe'", Customer.class).getSingleResult()
        );

        // Insert a new customer with a numeric phone number stored as a string
        emf.runInTransaction(em -> em.persist(new Customer(101L, "John Doe", "1234567890", "john.doe@jcg.com")));

        // Use cast() to treat the phoneNumber as an integer
        TypedQuery<Integer> query = emf.createEntityManager().createQuery(
                "select cast(c.phoneNumber as integer) from Customer c where c.id = 101", Integer.class
        );
        Integer result = query.getSingleResult();

        System.out.println("Phone number as integer: " + result);

        TypedQuery<String> query2 = emf.createEntityManager().createQuery(
                "select replace(c.email, 'oldmail.com', 'newmail.com') from Customer c where c.id = 5",
                String.class
        );
        String result2 = query2.getSingleResult();

        TypedQuery<Long> query3 = emf.createEntityManager().createQuery(
                "select id(c) from Customer c where c.email = 'example@domain.com'",
                Long.class
        );
        Long result3 = query3.getSingleResult();

        TypedQuery<String> leftQuery = emf.createEntityManager().createQuery(
                "select left(c.phoneNumber, 3) from Customer c where c.id = 7",
                String.class
        );
        String leftResult = leftQuery.getSingleResult();

        TypedQuery<String> rightQuery = emf.createEntityManager().createQuery(
                "select right(c.phoneNumber, 4) from Customer c where c.id = 7",
                String.class
        );
        String rightResult = rightQuery.getSingleResult();

        // Insert example data
        emf.runInTransaction(em -> {
            em.persist(new Customer(101L, "Alice", "alice@example.com", "123-456-7890"));
            em.persist(new Customer(102L, "bob", "bob@example.com", "234-567-8901"));
            em.persist(new Customer(103L, null, "null@example.com", "345-678-9012"));
            em.persist(new Customer(104L, "Charlie", "charlie@example.com", "456-789-0123"));
        });

        // Perform the sorted query
        TypedQuery<Customer> query4 = emf.createEntityManager().createQuery(
                "SELECT c FROM Customer c ORDER BY lower(c.name) ASC NULLS FIRST, c.id DESC",
                Customer.class
        );

        List<Customer> sortedCustomers = query4.getResultList();

        // Display the sorted result
        sortedCustomers.forEach(customers -> {
            System.out.println("Customer ID: " + customers.getId() +
                ", Name: " + customers.getName() +
                ", Email: " + customers.getEmail() +
                ", Phone Number: " + customers.getPhoneNumber());
        });

        if (customer != null) {
            System.out.println("Customer found: " + customer.getName() + ", " + customer.getEmail());
        } else {
            System.out.println("Customer not found.");
        }

        emf.close();

    }
}
